#include "units.h"
#include "warning.h"
#include <map>
#include <sstream>
#include <cstdlib>
using namespace std;

#define PUT(a) the_units[#a] = unit_rep(#a,a)

namespace blop
{
    namespace unit
    {
	string default_format_ = "[$%s$]";
	void default_format(const var &f) { default_format_ = f.str(); }
	string default_format() { return default_format_; }

	class unit_rep
	{
	public:
	    unit_rep() {}
	    unit_rep(const string s, double v) : symbol(s), value(v) {}
	    string symbol;
	    double value;
	};
	
	map<string,unit_rep> the_units;

	void put(const string &a, const string &b, double v)
	{
	    the_units[a] = unit_rep(b,v);
	}

	void fill_units()
	{
	    if(!the_units.empty()) return;

	    put("kilogram","kg",kg);
	    PUT(kg);
	    PUT(g);
	    PUT(amu);

	    put("second","s",s);
	    PUT(s);
	    PUT(ms);
	    put("mus","$\\mu$s",mus);
	    PUT(ns);
	    PUT(ps);
	    put("minute","min",minute);
	    put("min","min",minute);
	    PUT(hour);

	    put("Hertz","Hz",Hz);
	    put("hertz","Hz",Hz);
	    PUT(Hz);
	    PUT(kHz);
	    PUT(MHz);
	    PUT(GHz);

	    put("meter","m",meter);
	    PUT(m);
	    PUT(cm);
	    PUT(mm);
	    put("mum"   ,"$\\mu$m",mum);
	    put("micron","$\\mu$m",mum);
	    PUT(nm);
	    PUT(pm);
	    PUT(fm);
	    put("fermi","fm",fm);
	    put("angstrom","$\\AA$",angstrom);
	    PUT(km);
	    PUT(mile);
	    PUT(inch);
	    PUT(foot);
	    put("cm3","cm$^3$",cm3);
	    put("cm2","cm$^2$",cm2);
	    put("mm3","mm$^3$",mm3);
	    put("mm2","mm$^2$",mm2);
	 
	    put("Joule","J",J);
	    put("joule","J",J);
	    PUT(J);
	    PUT(mJ);
	    PUT(kJ);
	    PUT(MJ);
	    PUT(cal);
	    PUT(kcal);

	    put("Watt","W",W);
	    put("watt","W",W);
	    PUT(W);
	    PUT(kW);
	    PUT(MW);
	    PUT(GW);
	    PUT(mW);

	    put("Coulomb","C",C);
	    put("coulomb","C",C);
	    PUT(C);

	    put("Ampere","A",A);
	    put("ampere","A",A);
	    PUT(A);
	    PUT(mA);
	    put("muA","$\\mu$A",muA);
	    PUT(nA);
	    PUT(kA);

	    put("Newton","N",N);
	    put("newton","N",N);
	    PUT(N);

	    put("Kelvin","K",K);
	    put("kelvin","K",K);
	    PUT(K);
	    PUT(mK);

	    PUT(mole);
	    PUT(mol);

	    put("candela","cd",cd);
	    PUT(cd);

	    put("Volt","V",V);
	    put("volt","V",V);
	    PUT(V);
	    PUT(mV);
	    PUT(kV);
	    PUT(MV);

	    put("Tesla","T",T);
	    put("tesla","T",T);
	    PUT(T);
	    put("Gauss","G",G);
	    put("gauss","G",G);
	    put("kGauss","kG",kG);
	    put("kgauss","kG",kG);
	    PUT(G);
            PUT(kG);

	    PUT(Debye);

	    put("Ohm","$\\Omega$",Ohm);
	    put("ohm","$\\Omega$",Ohm);
	    put("mOhm","m$\\Omega$",mOhm);
	    put("muOhm","$\\mu\\Omega$",muOhm);
	    put("nOhm","n$\\Omega$",nOhm);
	    put("nanoOhm","n$\\Omega$",nOhm);
	    put("kOhm","k$\\Omega$",kOhm);
	    put("MOhm","M$\\Omega$",MOhm);

	    put("Siemens","S",S);
	    put("siemens","S",S);
	    PUT(S);
	    PUT(mS);
	    put("muS","$\\mu$S",muS);

	    put("radian","rad",rad);
	    PUT(rad);
	    PUT(mrad);
	    put("degree","deg",deg);
	    PUT(deg);

	    put("Farad","F",F);
	    put("farad","F",F);
	    PUT(F);
	    PUT(mF);
	    put("muF","$\\mu$F",muF);
	    put("nanoFarad","nF",nF);
	    put("nanofarad","nF",nF);
	    put("picoFarad","pF",pF);
	    put("picofarad","pF",pF);
	    PUT(nF);
	    PUT(pF);

	    put("Henry","H",H);
	    put("henry","H",H);
	    PUT(H);
	    PUT(mH);
	    put("muH","$\\mu$H",muH);
	    PUT(nH);
	    PUT(pH);

	    put("Pascal","Pa",Pa);
	    put("pascal","Pa",Pa);
	    PUT(Pa);
	    PUT(kPa);
	    PUT(MPa);
	    PUT(GPa);
	    PUT(mPa);
	    put("muPa","$\\mu$Pa",muPa);
	    PUT(Torr);
	    PUT(mTorr);
	    PUT(bar);
	    PUT(mbar);
	    PUT(atm);

	    put("atomic_mass","atomic\\_mass",atomic_mass);
	    put("atomic_charge","atomic\\_charge",atomic_charge);
	    put("atomic_length","atomic\\_length",atomic_length);
	    put("atomic_velocity","atomic\\_velocity",atomic_velocity);
	    put("atomic_time","atomic\\_time",atomic_time);
	    put("atomic_frequency","atomic\\_frequency",atomic_frequency);
	    put("atomic_energy","atomic\\_energy",atomic_energy);
	    put("atomic_dipole","atomic\\_dipole",atomic_dipole);
	    PUT(hartree);

	    PUT(eV);
	    PUT(keV);
	    PUT(MeV);
	    PUT(GeV);
	    PUT(TeV);

	    PUT(Ry);
	    put("Rydberg","Ry",Ry);
	}    

	bool find(const var &n, double *value, string *symbol)
	{
	    *value = 0;
	    if(symbol) *symbol = "";
	    
	    if(the_units.empty()) fill_units();

	    string name = replace(n,"*"," ").str();

	    // find the border between nominator and denominator: a / followed by space of
	    // a letter (i.e. a unit)
	    
	    string::size_type slash = string::npos;
	    for(string::size_type i=0; i<name.size()-1; ++i)
	    {
		if(name[i] == '/' && (name[i+1] == ' ' || isalpha(name[i+1])))
		{
		    slash = i;
		    break;
		}
	    }

	    string ss[2];
	    ss[0] = name;
	    ss[1] = "";
	    if(slash != string::npos)
	    {
		ss[0] = name.substr(0,slash);
		ss[1] = name.substr(slash+1,string::npos);
	    }
	    // now: ss[0] is the nominator, ss[1] is the denominator

	    *value = 1;
	    string symbol_local  = "";
	    for(int i=0; i<=1; ++i) // take both the nominator and the denominator
	    {
		if(i==1 && symbol_local == "")
		{
		    warning::print("Error in the unit: " & n);
		    *value = 0;
		    if(symbol) *symbol = "";
		    return false;
		}
		if(ss[i] == "") continue;
		if(i==1) symbol_local += '/';
		string e;
		istringstream str(ss[i]);
		while(str>>e)
		{
		    string unit_symbol = "";
		    double unit_value = 0;
		    string power_symbol = "";
		    double power_value  = 1;

		    if(isdigit(e[0]))
		    {
			char *endptr = 0;
			unit_symbol = e;
			unit_value = strtod(e.c_str(),&endptr);
			power_symbol = "";
			power_value = 1;
		    }
		    else
		    {
			string unitname  = e;  // unit name
			power_symbol = ""; // power
			string::size_type powpos = e.find("^");
			if(powpos != string::npos)
			{
			    unitname = e.substr(0,powpos);
			    power_symbol = e.substr(powpos+1,string::npos);
			}

			map<string,unit_rep>::iterator u = the_units.find(unitname);
			if(u == the_units.end())
			{
			    warning::print(string("Unit '") + unitname + string("' not found"));
			    *value = 0;
			    if(symbol) *symbol = "";
			    return false;
			}

			unit_symbol = (*u).second.symbol;
			unit_value  = (*u).second.value;

			power_value = 1;
			if(power_symbol != "")
			{
			    string::size_type powslash = power_symbol.find('/');
			    if(powslash == string::npos) 
			    {
				// no slash found, so simply convert it to double
				power_value = strtod(power_symbol.c_str(),0);
			    }
			    else
			    {
				// a slash found, so convert the nominator and denominator to double
				// separately, and divide them
				string p1 = power_symbol.substr(0,powslash);
				string p2 = power_symbol.substr(powslash+1,string::npos);
				power_value = strtod(p1.c_str(),0)/strtod(p2.c_str(),0);
			    }
			}
		    }
		    *value *= ::pow( unit_value, (i==0?1:-1)*power_value);
		    if(symbol_local != "" && symbol_local[symbol_local.size()-1] != '/') symbol_local += " ";
		    symbol_local += "\\mbox{" + unit_symbol + "}";
		    if(power_symbol != "") symbol_local += "^{" + power_symbol + "}";
		}
	    }
	    if(symbol) *symbol = symbol_local;
	    return true;
	}

	std::string symbol(const var &name)
	{
	    string sym="";
	    double val=0;
	    find(name,&val,&sym);
	    return sym;
	}
	double value(const var &name)
	{
	    double val=0;
	    find(name,&val);
	    return val;
	}


    }
}
